# Calculates Rt saturation following Ryerson and Watson (1987) and Hayden and Watson (2007) 
rtSaturation<-function(cats=milli,T=0,P=0,Ti=filterOut(WR,"Ti",1)){ # Little tested so far
    on.exit(options("show.error.messages"=TRUE))
    if(T==0){
        T<-winDialogString("Temperature (degrees C)","750")
        if(is.null(T)){cat("Cancelled.\n");options("show.error.messages"=FALSE);stop()}
    }
    T<-as.numeric(T)+273.15
    
    if(P==0){
        P<-winDialogString("Pressure (kbar)","3")
        if(is.null(P)){cat("Cancelled.\n");options("show.error.messages"=FALSE);stop()}
    }
    P<-as.numeric(P)
        
    cats<-cats[rownames(Ti),c("SiO2","TiO2","Al2O3","FeOt","MnO","MgO","CaO","Na2O","K2O","P2O5")]

    cats<-normalize2total(cats)/100
    
    FM<-1/cats[,"SiO2"]*(cats[,"Na2O"]+cats[,"K2O"]+2*(cats[,"CaO"]+cats[,"MgO"]+cats[,"FeOt"]))/cats[,"Al2O3"]
    
    #Ryerson and Watson (1987) 
    DTiO2<-exp(-3.16+9373/T+0.026*P-0.152*FM)
    
    #Ti.sat.RW<-599342.9/DTi
    Ti.sat.RW<-599342.9/DTiO2
    TiO2<-Ti/0.59950*1e-4
    TRt.sat.C.RW<-9373/(3.16+log(100/TiO2)-0.026*P+0.152*FM)-273.15
    #TRt.sat.C.RW<-9373/(3.16+log(599342.9/Ti)-0.026*P+0.152*FM)-273.15
    aTi.RW<-Ti/Ti.sat.RW
    
    #Hayden and Watson (2007)
    Ti.sat.HW<-10^(7.95-5305/T+0.124*FM)
    
    TRt.sat.C.HW<-5305/(7.95-log10(Ti)+0.124*FM)-273.15
    aTi.HW<-Ti/Ti.sat.HW
    
    y<-cbind(FM,Ti,round(Ti.sat.RW,1),round(aTi.RW,2),round(TRt.sat.C.RW,1),round(Ti.sat.HW,1),round(aTi.HW,2),round(TRt.sat.C.HW,1))
    colnames(y)<-c("FM","Ti.obs","Ti.sat.RW","aTi.RW","TRt.sat.C.RW","Ti.sat.HW","aTi.HW","TRt.sat.C.HW")
    if(!getOption("gcd.shut.up"))print(y)
    y<-formatResults(y)
    results<<-y
    return(y[,-2])
}
